import os
import numpy as np
from PytorchBoot.dataset import BaseDataset
import PytorchBoot.stereotype as stereotype

import sys
sys.path.append(r"C:\Document\Local Project\nbv_rec\nbv_reconstruction")

from utils.data_load import DataLoadUtil
from utils.pose import PoseUtil

@stereotype.dataset("nbv_reconstruction_dataset", comment="not tested")
class NBVReconstructionDataset(BaseDataset):
    def __init__(self, config):
        super(NBVReconstructionDataset, self).__init__(config)
        self.config = config
        self.label_dir = config["label_dir"]
        self.root_dir = config["root_dir"]
        self.datalist = self.get_datalist()

    def get_datalist(self):
        datalist = []
        scene_name_list = os.listdir(self.root_dir)
        for scene_name in scene_name_list:
            label_path = DataLoadUtil.get_label_path(self.label_dir, scene_name)
            label_data = DataLoadUtil.load_label(label_path)
            for data_pair in label_data["data_pairs"]:
                scanned_views = data_pair[0]
                next_best_view = data_pair[1]
                max_coverage_rate = label_data["max_coverage_rate"]
                datalist.append(
                    {
                        "scanned_views": scanned_views,
                        "next_best_view": next_best_view,
                        "max_coverage_rate": max_coverage_rate,
                        "scene_name": scene_name,
                    }
                )
        return datalist

    def __getitem__(self, index):
        data_item_info = self.datalist[index]
        scanned_views = data_item_info["scanned_views"]
        nbv = data_item_info["next_best_view"]
        max_coverage_rate = data_item_info["max_coverage_rate"]
        scene_name = data_item_info["scene_name"]
        scanned_views_pts, scanned_coverages_rate, scanned_cam_pose = [], [], []
        for view in scanned_views:
            frame_idx = view[0]
            coverage_rate = view[1]
            view_path = DataLoadUtil.get_path(self.root_dir, scene_name, frame_idx)
            pts = DataLoadUtil.load_depth(view_path)
            scanned_views_pts.append(pts)
            scanned_coverages_rate.append(coverage_rate)
            cam_pose = DataLoadUtil.load_cam_info(view_path)["cam_to_world"]
            
            cam_pose_6d = PoseUtil.matrix_to_rotation_6d_numpy(np.asarray(cam_pose[:3,:3]))
            translation = cam_pose[:3,3]
            cam_pose_9d = np.concatenate([cam_pose_6d, translation], axis=0)
            scanned_cam_pose.append(cam_pose_9d)

        nbv_idx, nbv_coverage_rate = nbv[0], nbv[1]
        nbv_path = DataLoadUtil.get_path(self.root_dir, scene_name, nbv_idx)
        nbv_pts = DataLoadUtil.load_depth(nbv_path)
        cam_info = DataLoadUtil.load_cam_info(nbv_path)
        nbv_cam_pose = cam_info["cam_to_world"]
        nbv_cam_pose_6d = PoseUtil.matrix_to_rotation_6d_numpy(np.asarray(nbv_cam_pose[:3,:3]))
        translation = nbv_cam_pose[:3,3]
        nbv_cam_pose_9d = np.concatenate([nbv_cam_pose_6d, translation], axis=0)
        data_item = {
            "scanned_views_pts": np.asarray(scanned_views_pts,dtype=np.float32),
            "scanned_coverages_rate": np.asarray(scanned_coverages_rate,dtype=np.float32),
            "scanned_cam_pose": np.asarray(scanned_cam_pose,dtype=np.float32),
            "nbv_pts": np.asarray(nbv_pts,dtype=np.float32),
            "nbv_coverage_rate": nbv_coverage_rate,
            "nbv_cam_pose": nbv_cam_pose_9d,
            "max_coverage_rate": max_coverage_rate,
            "scene_name": scene_name
        }

        return data_item

    def __len__(self):
        return len(self.datalist)

if __name__ == "__main__":
    import torch
    config = {
        "root_dir": "C:\\Document\\Local Project\\nbv_rec\\sample_dataset",
        "label_dir": "C:\\Document\\Local Project\\nbv_rec\\sample_output",
        "ratio": 0.1,
        "batch_size": 1,
        "num_workers": 0,
    }
    ds = NBVReconstructionDataset(config)
    dl = ds.get_loader(shuffle=True)
    for idx, data in enumerate(dl):
        for key, value in data.items():
            if isinstance(value, torch.Tensor):
                print(key, ":" ,value.shape)
        print()